module casc
 implicit none
 character*256 output_file, decay_file, init_file, rel_path, gle_file, general_file, wk_file
 character*20  element_names(0:200)
 double precision time_cut, lambda_cut, it_probability, kt, effective_decay_rate(5), max_kt, max_beta, min_beta
 integer number_reaction_types, number_levels, number_isomers, number_intermediate
 integer a, z, n																	! A, Z, N of isotope under investigation
 integer,          dimension (:)     ,allocatable,target :: z_product			    ! Charge of reaction product nucleus
 integer,          dimension (:)     ,allocatable,target :: n_product			    ! Neutron number of reaction product nucleus
 double precision, dimension (:)     ,allocatable,target :: e_product			    ! Energy level in product nucleus
 double precision, dimension (:)     ,allocatable,target :: e_level			    	! Energy level of state (keV)
 double precision, dimension (:)     ,allocatable,target :: g_level			    	! degeneracy of level
 double precision, dimension (:)     ,allocatable,target :: j_level			    	! angular momentum J of level (h_bar)
 double precision, dimension (:)     ,allocatable,target :: p_level			    	! parity of level (+-1)
 double precision, dimension (:)     ,allocatable,target :: lambda			    	! terrestrial decay constant of state (1/s) 
 double precision, dimension (:)     ,allocatable,target :: stellar_renormalization_factor			    	! terrestrial / (terrestrial + thermal excitation) 
 double precision, dimension (:,:)   ,allocatable,target :: transition				! terrestrial probability of internal transition as a function of initial level and final level
 double precision, dimension (:,:)   ,allocatable,target :: decay					! terrestrial probability of decay as a function of initial level and reaction channel
 double precision, dimension (:,:)   ,allocatable,target :: lambda_transition		! terrestrial internal transition rate as a function of initial level and final level
 double precision, dimension (:,:)   ,allocatable,target :: lambda_transition_wk	! Weisskopf prediction of terrestrial internal transition rate as a function of initial level and final level
 double precision, dimension (:,:)   ,allocatable,target :: lambda_decay			! terrestrial decay rate as a function of initial level and reaction channel
 double precision, dimension (:,:)   ,allocatable,target :: lambda_kt				! thermal excitation rate as a function of initial level and final level
 double precision, dimension (:,:)   ,allocatable,target :: lambda_transition_effective		! effective stellar internal transition rate as a function of initial and final long-lived (!!) level
 double precision, dimension (:,:)   ,allocatable,target :: lambda_decay_effective			! effective stellar decay rate as a function of initial long-lived level and reaction channel
 integer,          dimension (:,:)   ,allocatable,target :: output_unit_decay	    ! unit to which the output is written to
 integer,          dimension (:,:)   ,allocatable,target :: output_unit_transition  ! unit to which the output is written to
 character*256,    dimension (:,:)   ,allocatable,target :: transition_output_file,decay_output_file  ! file to which the output is written to
 integer output_unit_generals(20)	    ! unit to which the general output is written to
 logical wk_completion
 integer internal_level_id(0:1000000)												! gives internal level-id as a function of the external ID (external ID must be inside [0,1000000])
 integer internal_reaction_id(100)													! gives internal reaction id as a function of the external ID (external ID must be inside [1,100])
 integer unit_counter
!
!
end module casc

program make_casc
 use casc
 implicit none
 integer kt_int, i
!
 wk_completion = .true.
 call initfiles 
 print*,'**************** initfiles ok' 
 call initdecay 
 print*,'**************** initdecay ok' 
 call initoutput
 print*,'**************** initoutput ok' 
 do kt_int=1,1000
! do kt_int=1,100
   kt = kt_int
   call calculate_rates
   call output
 end do  
 do kt_int=110,100,10
   kt = kt_int
   call calculate_rates
   call output
 end do  
 call make_general_output
 call make_gle
call close_files
end 


subroutine output
 use casc
 implicit none
 integer i,j, uc, max_level,above_cut_counter
 logical below_cut
!
 below_cut = .true.
 above_cut_counter = 0
 do i=1, number_levels
   do j=i+1, number_levels
     if (lambda(i) .ge. lambda_cut) then                              ! short-lived state
       below_cut = below_cut .and. ( 10.*lambda_kt(i,j) .le. lambda(i))
       if ( 10.*lambda_kt(i,j) .gt. lambda(i)) above_cut_counter = above_cut_counter + 1
     end if  
   end do
 end do
 if (above_cut_counter .le. 1 .and. kt .gt. max_kt) max_kt = kt
! print*,below_cut, kt, max_kt, minval(stellar_renormalization_factor), minloc(stellar_renormalization_factor) &
!     , above_cut_counter
 do i=1, number_levels
   if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
     do j=1, number_reaction_types         
       uc = output_unit_decay(i,j)
       write(uc, *) kT, lambda_decay_effective(i,j)
       if (kt.le.max_kt) max_beta = max(lambda_decay_effective(i,j),max_beta)
       if (kt.le.max_kt) min_beta = min(lambda_decay_effective(i,j),min_beta)
!       max_beta = max(lambda_decay_effective(i,j),max_beta)
!       min_beta = min(lambda_decay_effective(i,j),min_beta)
     end do
     do j=1, number_levels         
       if (lambda(j) .lt. lambda_cut .and. i.ne.j) then   			! isomer/gs
         uc = output_unit_transition(i,j)
         write(uc, *) kT, lambda_transition_effective(i,j)
       end if  
     end do
   end if  
 end do
 uc = output_unit_decay(0,0)
 write(uc, *) kT, effective_decay_rate(3), dlog(2.d0)/effective_decay_rate(3), effective_decay_rate(4), effective_decay_rate(5)
end     

subroutine close_files
 use casc
 implicit none
 integer i
!
 do i=20,unit_counter-1
   close(i)
 end do
end 

subroutine initoutput
 use casc
 implicit none
 integer i,j
 character*3 c3
 character*7 c_end
!
 c_end='.dat'
 if (wk_completion) c_end = '_wk' //trim(c_end)
!
! decays
!
 do i=1, number_levels
   if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
     do j=1, number_reaction_types         
        write(c3,'(I3.3)') a				
        output_file = c3 //trim(element_names(z))
        write(c3,'(I3.3)') i				
        output_file = trim(output_file) // '_level_'//c3
        write(c3,'(I3.3)') z_product(j)+n_product(j)				
        output_file = trim(output_file) // '_to_'// c3 // trim(element_names(z_product(j))) &
                      // '_effective_decay' // trim(c_end)
        decay_output_file(i,j) = output_file
        output_file = trim(rel_path) // decay_output_file(i,j)
        print*,'Output file: ',trim(output_file)
        open(unit=unit_counter, file=output_file, status='unknown')
        write(unit_counter, *) 'kT (keV)             effective decay rate (1/s)'
        output_unit_decay(i,j) = unit_counter
        unit_counter=unit_counter+1
     end do
   end if  
 end do
!
! internal transitions
!
 do i=1, number_levels
   if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
     do j=1, number_levels         
       if (lambda(j) .lt. lambda_cut .and. i.ne.j) then   			! isomer/gs
         write(c3,'(I3.3)') a				
         output_file = c3 //trim(element_names(z))
         write(c3,'(I3.3)') i				
         output_file = trim(output_file) // '_level_'//c3
         write(c3,'(I3.3)') j				
         output_file = trim(output_file) // '_to_level_'// c3 // '_effective_transition' // trim(c_end)
         transition_output_file(i,j) = output_file
         output_file = trim(rel_path) // transition_output_file(i,j)
         print*,'Output file: ',trim(output_file)
         open(unit=unit_counter, file=output_file, status='unknown')
         write(unit_counter, *) 'kT (keV)             effective transition rate (1/s)'
         output_unit_transition(i,j) = unit_counter
         unit_counter=unit_counter+1
       end if  
     end do
   end if  
 end do
!
!  effective lifetime of isotope
! 
 write(c3,'(I3.3)') a				
 output_file = trim(rel_path) // c3 //trim(element_names(z)) // '_effective_lifetime' // trim(c_end)
 print*,'Output file: ',trim(output_file)
 open(unit=unit_counter, file=output_file, status='unknown')
 write(unit_counter, *) 'kT (keV)             effective decay rate of isotope (1/s)    effective half life of isotope (s)'
 output_unit_decay(0,0) = unit_counter
 unit_counter=unit_counter+1
!
!  gle-output
! 
 write(c3,'(I3)') a			
               i=1
 if (a.lt.100) i=2	
 if (a.lt.10)  i=3	
 gle_file = trim(rel_path) // trim(element_names(z)) // c3(i:3) //'_1_2.gle'
 print*,'Gle file 1-2: ',trim(gle_file)
 open(unit=unit_counter, file=gle_file, status='unknown')
 output_unit_generals(2) = unit_counter
 unit_counter=unit_counter+1
! 
 gle_file = trim(rel_path) // trim(element_names(z)) // c3(i:3) //'_beta.gle'
 print*,'Gle file beta: ',trim(gle_file)
 open(unit=unit_counter, file=gle_file, status='unknown')
 output_unit_generals(3) = unit_counter
 unit_counter=unit_counter+1
! 
end

subroutine calculate_rates
 use casc
 implicit none
 integer i,j,k,l
 double precision ll, sum_k, sum_l
!
 call calculate_thermal_excitation 
!
! effective transitions between long-lived states
!
 lambda_transition_effective = 0.
 do i=1, number_levels
   if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
     do j=1, number_levels          				
       if (lambda(j) .lt. lambda_cut .and. j.ne.i) then  		! isomer/gs
         ll = lambda_kT(i,j) + lambda_transition(i,j)
         sum_l=0.
         do l=1, number_levels
           if (lambda(l) .ge. lambda_cut) then   			! intermediate state
             sum_k=0.
             do k=1,l-1
               if (lambda(k) .ge. lambda_cut) then   			! intermediate state
                 sum_k = sum_k + transition(l,k)*transition(k,j)*stellar_renormalization_factor(k)
               end if
             end do
             sum_l = sum_l + lambda_kt(i,l)*(transition(l,j)*stellar_renormalization_factor(l)+sum_k) 
           end if 
         end do
         lambda_transition_effective(i,j) = ll+sum_l
       end if
     end do
   end if
 end do  
!
! effective decays of long-lived states
!
 lambda_decay_effective = 0.
 do i=1, number_levels
   if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
     do j=1, number_reaction_types          				
       ll = lambda_decay(i,j)
       sum_l=0.
       do l=1, number_levels
         if (lambda(l) .ge. lambda_cut) then   			! intermediate state
           sum_k=0.
           do k=1,l-1
             if (lambda(k) .ge. lambda_cut) then   			! intermediate state
               sum_k = sum_k + transition(l,k)*decay(k,j)*stellar_renormalization_factor(k)
             end if
           end do
           sum_l = sum_l + lambda_kt(i,l)*(decay(l,j)*stellar_renormalization_factor(l)+sum_k) 
         end if 
       end do
       lambda_decay_effective(i,j) = ll+sum_l
     end do
   end if
 end do  
 call calculate_effective_decay_rate 
end

subroutine calculate_thermal_excitation
 use casc
 implicit none
 integer i,j
 double precision renorm_sum
!
 lambda_kt = 0.
 do i=1,number_levels
   renorm_sum = 0.
   do j=1,i-1
     if (lambda_transition(i,j) .gt. 0.) then
       lambda_kt(j,i) = lambda_transition(i,j) * g_level(i)/g_level(j)
       lambda_kt(j,i) = lambda_kt(j,i) * dexp((e_level(j)-e_level(i))/kt)
     end if  
   end do
 end do
 do i=1,number_levels
   renorm_sum = sum(lambda_kt(i,:))
   stellar_renormalization_factor(i) = lambda(i) / (lambda(i) + renorm_sum)
 end do  
end

subroutine initfiles
 use casc
 implicit none
 character*300 buffer
 integer iostats, i, j
!
 print*
 call getarg(1,buffer)
 if (buffer.eq.'-') then	 
   print*,'Kein ini file!'
   init_file=''
   call getarg(2,buffer)
   decay_file='levels/' // buffer(1:len_trim(buffer))
   call getarg(3,buffer)
   read(buffer,*) time_cut
   call getarg(4,buffer)
   rel_path = 'output/' // buffer(1:len_trim(buffer)) // '/'
 else 
   if (buffer.eq.'') then	 
     write(*,'(A30)', advance='NO') 'Initfile           : '
     read(*,*) init_file
   else 
     init_file=buffer(1:len_trim(buffer))
   end if 	 
!
   if (init_file(len_trim(init_file)-3:len_trim(init_file)).eq.'.sch') then
     decay_file = init_file(1:len_trim(init_file))
     write(*,*) 'Level scheme file  : ', trim(decay_file)
     rel_path = 'output/'
     write(*,*) 'Relative path      : ', trim(rel_path)
     open(unit=13,file=decay_file,status='unknown')
       read(13,*) i,j,time_cut
       write(*,*) 'Isomer time cut    : ', time_cut, ' s'
     close(13)
   else
     print*, 'Initfile: ',init_file
     open(unit=13,file=init_file,status='unknown')
       read(13,*) rel_path
       write(*,*) 'Relative path      : ', trim(rel_path)
       read(13,*) decay_file
       write(*,*) 'Level scheme file  : ', trim(decay_file)
       decay_file = trim(rel_path) // '/' // trim(decay_file)
       read(13,*) time_cut
       write(*,*) 'Isomer time cut    : ', time_cut, ' s'
     close(13)
   end if
 end if  
 lambda_cut = dlog(2.d0)/time_cut
 open(unit=13,file='element_symbols.txt',status='old')
   iostats=0
   i=0
   element_names(0) = 'NN'
   do while (iostats.eq.0)
     i=i+1
     read(13,*,iostat=iostats) element_names(i)
   end do 
 close(13)   
 unit_counter=20
!
!  general-output
! 
 general_file = trim(rel_path) // 'general_information.txt'
 print*,'General output file: ',trim(general_file)
 open(unit=unit_counter, file=general_file, status='unknown')
 output_unit_generals(1) = unit_counter
 unit_counter=unit_counter+1
!
!  weisskopf-output
! 
 wk_file = trim(rel_path) // 'weisskopf_approximation.txt'
 print*,'Weisskopf file: ',trim(wk_file)
 open(unit=unit_counter, file=wk_file, status='unknown')
 output_unit_generals(4) = unit_counter
 unit_counter=unit_counter+1
!
end

subroutine initdecay
 use casc
 implicit none
 integer level, uc
!
 internal_level_id    = -1
 internal_reaction_id = -1
 call initlevels
 allocate (lambda_transition(number_levels,number_levels))
 allocate (lambda_transition_effective(number_levels,number_levels))
 allocate (lambda_kt(number_levels,number_levels))
 allocate (lambda_decay(number_levels,number_reaction_types))
 allocate (lambda_decay_effective(number_levels,number_reaction_types))
!
 allocate (output_unit_decay(0:number_levels,0:number_reaction_types+20))
 allocate (output_unit_transition(number_levels,number_levels))
 allocate (transition_output_file(number_levels,number_levels))
 transition_output_file = ''
 allocate (decay_output_file(number_levels,number_reaction_types))
 decay_output_file = ''
!
 call init_wk
!
 do level=1,number_levels
   lambda_transition(level,:) = lambda(level) * transition(level,:)
   lambda_decay(level,:)      = lambda(level) * decay(level,:)
 end do
!
!
 call get_rates_wk
!
 uc = output_unit_generals(4)
 write(uc,*) 
 write(uc,*) 'General level information:'
 do level=1,number_levels
   if (lambda(level) .le. lambda_cut) then
     number_isomers      = number_isomers + 1
     write(uc,*) 'Level ',level,' - isomer, half-life time: ',0.693/lambda(level),' s'
   else
     number_intermediate = number_intermediate + 1
     write(uc,*) 'Level ',level,' - intermediate state, half-life time: ',0.693/lambda(level),' s'
   end if  
 end do
!
 write(uc,*) 'Number of long-lived states  :', number_isomers
 write(uc,*) 'Number of intermediate states:', number_intermediate
 max_kt = 0.
 min_beta = 1.d15
 max_beta = 1.d-20
! stop
end 

subroutine initlevels
 use casc
 implicit none
 integer i,j, reaction_id, dz, dn, level_id
 double precision level_energy, level_j, t12, branching, norm, level_p
 integer nbr_target_levels, io
!
 open(unit=12,file=decay_file,status='old')
   read(12,*) z, n
   a = z+n
   print *,'A, Z, N: ', a, z, n 
   read(12,*) number_reaction_types
   print *,'Number of reaction types: ', number_reaction_types
   if (number_reaction_types.gt.0) then
     allocate (z_product(number_reaction_types))
     allocate (n_product(number_reaction_types))
     allocate (e_product(number_reaction_types))
     do i=1,number_reaction_types
       read(12,*) reaction_id, dz, dn, level_energy
       if (reaction_id .eq. 0) then
         print*,'Reaction-ID must not be 0 - this is reserved for internal transition. Check input.'
         stop
       end if  
       if (internal_reaction_id(reaction_id) .eq. -1) then
         internal_reaction_id(reaction_id) = i
       else
         print*,'Reaction-ID ',reaction_id, ' not unique. Check input.'
         stop
       end if  
       z_product(i) = z + dz
       n_product(i) = n + dn
       e_product(i) = level_energy
 !      print*,reaction_id, z_product(internal_reaction_id(reaction_id)), n_product(internal_reaction_id(reaction_id)), &
 !         level_energy,internal_reaction_id(reaction_id)
     end do
     read(12,*) number_levels
	 print*,'Number of levels:', number_levels
	 allocate (e_level(number_levels))
	 allocate (g_level(number_levels))
	 allocate (j_level(number_levels))
	 allocate (p_level(number_levels))
	 allocate (lambda(number_levels))
	 allocate (stellar_renormalization_factor(number_levels))
	 allocate (transition(number_levels,number_levels))
	 allocate (decay(number_levels,number_reaction_types))
     allocate (lambda_transition_wk(number_levels,number_levels))
     do i=1,number_levels
	   read(12,*,iostat=io) level_id, level_energy, level_j, level_p, t12, nbr_target_levels
       if (io.ne.0) then
         print*,'Problem reading the level header line'
         print*,'Tried to read the ',i,'th level'
         print*,level_id, level_energy, level_j, level_p, t12, nbr_target_levels
         stop
       end if
       if (internal_level_id(level_id) .eq. -1) then
         internal_level_id(level_id) = i
       else
         print*,'Level-ID ',level_id, ' not unique. Check input.'
         stop
       end if  
       e_level(i) = level_energy
       j_level(i) = level_j
       p_level(i) = level_p
       g_level(i) = 2.*j_level(level_id) + 1.
       if (t12 .le. 0.) then
         lambda(i)  = -1.d0
       else  
         lambda(i)  = dlog(2.d0) / t12
       end if  
       norm = 0.
       do j=1, nbr_target_levels
         read(12,*) reaction_id, level_id, branching
           if (nbr_target_levels .eq. 1) branching=1.d0
           if (reaction_id .eq. 0) then  !  IT
             if (internal_level_id(level_id) .gt. -1) then
               transition(i,internal_level_id(level_id)) = branching
             end if  
           else if  (reaction_id .eq. 1) then  !  Reaction changing N,Z
             if (internal_reaction_id(level_id) .eq. -1) then
               print*,'Reaction-ID ',level_id , ' not defined. Check input.'
               stop
             end if  
             decay(i,internal_reaction_id(level_id)) = branching
           else
             print*,'something wrong with input. Type can only be 0 or 1, not ',  reaction_id
             print*,'Level',i,level_id
             stop
           end if
           norm = norm + branching
       end do
       if (norm.eq.0. .and. nbr_target_levels.gt.0) then
         print*,'Something wrong with branchings - sum equals zero for level:', i, level_id
       else 
         if (norm.gt.0.) then
           transition(i,:) = transition(i,:) / norm
           decay(i,:)      = decay(i,:) / norm
         end if  
       end if  
     end do
   end if
 close(12)
 do i=2, number_levels
   if (e_level(i).le.e_level(i-1)) then
     print*,'Please sort levels according to energy.'
     stop
   end if  
 end do 
end

subroutine init_wk
 use casc
 implicit none
 integer i,j,uc
 double precision norm
!
! print*,'was here --------------------init_wk'
 uc = output_unit_generals(4)
 do i=1,number_levels
   do j=i-1,1,-1
     call get_weisskopf(i,j)
   end do
   if (lambda(i) .eq. -1.d0) then
     write(uc,*) 'Will assume Weisskopf approximation for half-live of state', i
     lambda(i) = sum(lambda_transition_wk(i,:))
     write(uc,*) 'Weisskopf - half life time:', dlog(2.d0)/lambda(i), ' s'
   end if
 end do
end 

subroutine get_rates_wk
 use casc
 implicit none
 integer i,j, uc
 double precision norm
!
! print*,'was here --------------------get_rates_wk'
 uc = output_unit_generals(4)
 do i=1,number_levels
   do j=i-1,1,-1
     if (transition(i,j) .eq. 0.d0) then 
       lambda_transition(i,j) = lambda_transition_wk(i,j)
       write(uc,*) 'Will assume Weisskopf approximation for transition of state', i, ' to state',j
       write(uc,*) 'Weisskopf - half life time:', dlog(2.d0)/lambda_transition_wk(i,j), ' s'
     end if  
   end do
   norm = sum(lambda_transition(i,:)) + sum(lambda_decay(i,:))
   transition(i,:) = lambda_transition(i,:)/norm
 end do
end 

subroutine get_weisskopf(i,j)
 use casc
 implicit none
 integer i,j, reaction_id, dz, dn, level_id, ll, hl
 double precision energy,l, h, c, alpha, mp, wk, df, hwz,r, r0
 logical l_even, dp, ttype
!
 ll = min(i,j)
 hl = max(i,j)
 h     = 6.58211d-22                     ! MeV s
 c     = 299792458.d0                    ! m/s
 alpha = 1./137.                         ! cgs
 mp    = 938.3/c**2                      ! MeV*s^2/m^2
 r0    = 1.26d0                          ! fm
 r = r0*(a**(1.d0/3.D0))/1.d15           ! m
 energy = ( e_level(hl) - e_level(ll) )  / 1.d3 
 l      = dabs(j_level(hl) - j_level(ll))
 if (l.gt.0) then
! print*,'Transition energy:', energy, ' MeV'
 l_even = (modulo(l,2.d0) .eq. 0.d0)
 dp     = (p_level(ll)*p_level(hl) .gt. 0.d0)      				! dp = true, if no parity change, dp = false if parity change
 ttype  = (l_even .and. dp ) .or. (.not.l_even .and. .not.dp)	! dp = true if E-transition, dp = false if M-transition
 if (ttype) then
!   print*,'Transition multipolarity:  E', int(l)
   wk = h*((2.*(l+1.))/(l*(df(2.*l+1.))**2))*(9./(l+3.)**2)*((energy/(h*c))**(2.*l+1.))* &
        alpha*c*(R**(2.*l)) ! MeV
 else 
!   print*,'Transition multipolarity:  M', int(l)
   wk = h*((20.d0*(l+1.))/(l*(df(2.*l+1.))**2))*(9./(l+3.)**2)*((energy/(h*c))**(2.*l+1.))* &
        alpha*c*(R**(2.*l-2.))*((h/(mp*c))**2) ! MeV
 end if
! print*,'Weisskopf unit: ', wk,' MeV'
! hwz = dlog(2.d0)*h/wk
! print*,'Half life time: ', hwz, ' s'
 lambda_transition_wk(i,j) = wk/h
 else
   lambda_transition_wk(i,j) = 0.
 end if  
end

recursive double precision function df(n) result(res)
implicit none
double precision n
!
if (n.lt.2) then
  res = 1.d0
else
  res = n*df(n-2.d0)
end if
end      

subroutine calculate_effective_decay_rate
 use casc
 implicit none
 integer i,j, s, l
 double precision lls, lsl, lbs, lbl, a1, p, q, d,x1,x2
!
 effective_decay_rate = 0.
 if (number_isomers .ge.2) then
   s = 0                                    ! short
   l = 0									! long
   do i=1,number_levels
     if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
       if (s.gt.0) then
         if (lambda(i) .lt. lambda(s)) then
           l=i
         else
           l=s
           s=i
         end if
         exit
       else
         s=i
       end if
     end if          
   end do
 lsl = lambda_transition_effective(s,l)
 lls = lambda_transition_effective(l,s)
 lbl = sum(lambda_decay_effective(l,:))
 lbs = sum(lambda_decay_effective(s,:))
 a1  = lls/lsl
! print*,'s,l,lsl,lls,lbl,lbs',s,l,lsl,lls,lbl,lbs
! 
! rough approximation
!
 effective_decay_rate(1) = lbl+lls*lbs/(lbs+lsl)
 effective_decay_rate(2) = (lbs*a1+lbl)/(1+a1)
 effective_decay_rate(3) = min(effective_decay_rate(1),effective_decay_rate(2))
!
! exact solution of differential equation
! 
 p = lsl+lbs+lls+lbl
 q = (lsl+lbs)*(lls+lbl)-lls*lsl
!
! eigenvalues are the solution of the equation x*x+p*x+q=0
! 
 d = p*p/4.d0 -q
 if (d.lt.0) then
   print*,'no solution to DGL', kT
 else
   d  = dsqrt(d)
   x1 = -p/2.d0 + d    
   x2 = -p/2.d0 - d    
 end if
!
!  we want -lambda instead of Eigenvalue
! 
 effective_decay_rate(4) = min(-x1,-x2)
 effective_decay_rate(5) = max(-x1,-x2)
 else if (number_isomers .eq. 1) then
   do i=1,number_levels
     if (lambda(i) .lt. lambda_cut) then   			! isomer/gs
       s=i
       exit
     end if          
   end do
   effective_decay_rate =   sum(lambda_decay_effective(s,:)) 
 else
   print*,l,s,number_isomers
   effective_decay_rate = 0
 end if
end

subroutine make_gle
 use casc
 implicit none
 integer uc,i,j,lev,zprod,min_y, max_y
 character*3 c3
 character*4 c4
 logical hit
 real max_x
!
 max_x = min(100.,max_kt+10)
 max_x = max(30.,max_x)
!
!  transitions between first 2 long-lived levels
!
 uc = output_unit_generals(2)
 write(uc, *) 'size 20 12'
 write(uc, *) 'amove 3 2'
 write(uc, *) 'begin graph'
 write(uc, *) '  nobox'
 write(uc, *) '  fullsize'
 write(uc, *) '  size 14 8.'
 write(uc, *) '   title "^{',a,'}',trim(element_names(z)),' - internal transitions" hei 0.7'
 write(uc, *) '   ytitle "\lambda_{IT} (1/s)" hei 0.6'
 write(uc, *) '   xtitle "kT (keV)" hei 0.6'
 write(uc, *) '   xaxis min 1 max',max_x
 write(uc, *) '   yaxis min 1e-20 max 1e10 log'
 write(uc, *) '   x2axis off'
 write(uc, *) '   y2axis off'
 write(uc, *) '   xlabels hei 0.6'
 write(uc, *) '   ylabels hei 0.6'
 write(uc, *) '   key pos tl hei 0.6'
 hit = .false.
 do i=1,number_levels
   do j=i+1,number_levels
     hit = (transition_output_file(i,j) .ne. '') 
     if (hit) exit
   end do
   if (hit) exit
 end do    
 if (hit) then
   write(uc, *) '   data ',trim(transition_output_file(i,j)),' d4=c1,c2 ignore 1'
   write(uc, *) '   data ',trim(transition_output_file(j,i)),' d6=c1,c2 ignore 1'
   write(uc, *) '   d6 line lstyle 1 lwidth 0.1 color firebrick key "^{',a,'}',element_names(z),&
                ', IT: Iso \rightarrow GS"'
   write(uc, *) '   d4 line lstyle 4 lwidth 0.1 color black key     "^{',a,'}',element_names(z),&
                ', IT: GS \rightarrow Iso"'
 end if  
 write(uc, *) 'end graph'
!
! decay of first long-lived state
!
 uc = output_unit_generals(3)
 write(uc, *) 'size 20 12'
 write(uc, *) 'amove 3 2'
 write(uc, *) 'begin graph'
 write(uc, *) '  nobox'
 write(uc, *) '  fullsize'
 write(uc, *) '  size 14 8.'
 write(uc, *) '   title "^{',a,'}',trim(element_names(z)),' - \beta - decays"  hei 0.7'
 write(uc, *) '   ytitle "\lambda_{\beta} (1/s)"  hei 0.6'
 write(uc, *) '   xtitle "kT (keV)"  hei 0.6'
 write(uc, *) '   xaxis min 1 max',max_x
! write(uc, *) '   yaxis min 1e-20 max 1e5 log'
 min_y = int(dlog(min_beta)/dlog(10.d0))-1
 max_y = int(dlog(max_beta)/dlog(10.d0))+1
 write(uc, *) '   yaxis min',real(10.**min_y),' max',real(10.**max_y),' log'
 write(uc, *) '   xlabels hei 0.6'
 write(uc, *) '   ylabels hei 0.6'
 write(uc, *) '   x2axis off'
 write(uc, *) '   y2axis off'
 write(uc, *) '   key pos tl hei 0.6'
 hit = .false.
 do lev=1,number_levels
   do j=1,number_reaction_types
     if (decay_output_file(lev,j) .ne. '') then  ! need to check, if indeed beta-decay ...
       hit = ((z-z_product(j))*(n-n_product(j)) .eq. -1) 
     end if
     if (hit) exit
   end do
   if (hit) exit
 end do    
 if (hit) then
   do j=1,number_reaction_types
     if (decay_output_file(lev,j) .ne. '' .and. (z-z_product(j))*(n-n_product(j)) .eq. -1) then
       if (j.lt.100) write(c4,'(A2,I2)') ' d',j
       if (j.lt.10)  write(c4,'(A2,I1)') ' d',j	
       write(uc, *) '   data ',trim(decay_output_file(lev,j)),c4,'=c1,c2 ignore 1'
       write(uc, *) c4,' line lstyle' ,j, ' lwidth 0.1 color black key     "^{',a,'}',trim(element_names(z)),&
                ' \rightarrow ^{',a,'}',trim(element_names(z_product(j))),'"'
     end if           
   end do             
 end if  
 write(uc, *) 'end graph'
end

subroutine make_general_output
 use casc
 implicit none
 integer uc, max_kt_int
!
 uc = output_unit_generals(1)
 max_kt_int = max_kt
 write(uc, *) 'Total number of states                          :', number_levels 
 write(uc, *) 'Number of long-lived states                     :', number_isomers 
 write(uc, *) 'Number of intermediate states                   :', number_intermediate
 write(uc, *) 'Isomer cut (s)                                  :', time_cut
 write(uc, *) 'Maximum temperature with valid assumptions (keV):', max_kt_int
!
 print*,'Maximum temperature with valid assumptions: ', max_kt_int, ' keV'
!
end

